<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Staudenmeir\EloquentHasManyDeep\HasRelationships;

class Client extends Model
{
    use SoftDeletes;
    use HasRelationships;

    protected $fillable = [
        'name',
        'about',
        'email',
        'phone',
        'avatar'
    ];

    public function projects ()
    {
        return $this->hasMany('App\Project');
    }

    public function milestones ()
    {
        return $this->hasManyThrough('App\Milestone', 'App\Project');
    }

    public function invoices ()
    {
        return $this->hasManyDeep(
            'App\Invoice',
            [ 'App\Project', 'App\Milestone' ]
        );
    }

    public function payments ()
    {
        return $this->hasManyDeep(
            'App\Payment',
            [ 'App\Project', 'App\Milestone', 'App\Invoice' ]
        );
    }
}
