<?php

namespace App\Http\Controllers\Helpers;

use App\User;
use App\Invoice;
use Mpdf\Mpdf as PDF;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Storage;
use Mpdf\Output\Destination as PDFDestination;

class InvoiceStorageHelper
{
    public $id;
    private static $invoice;

    public function __construct()
    {
        $this->id = Str::uuid();
    }

    public function path()
    {
        $path = 'invoices/'.$this->id.'.pdf';
        return $path;
    }

    public function url()
    {
        $url = Storage::url($this->path());
        return $url;
    }

    public function pay()
    {
        $route = 'pay/'.$this->id;
        return url($route);
    }

    private static function template()
    {
        $data = [
            'invoice' => self::$invoice,
            'milestone' => self::$invoice->milestone,
            'project' => self::$invoice->milestone->project,
            'client' => self::$invoice->milestone->project->client,
            'user' => User::firstOrFail()
        ];
        $template = view('templates.invoice', $data)->render();
        return $template;
    }

    private static function file()
    {
        $pdf = new PDF;
        $pdf->Bookmark('Invoice #'.self::$invoice->id);
        $pdf->useSubstitutions = false;
        $pdf->simpleTables = true;
        $pdf->WriteHTML(self::template());
        $file = $pdf->Output(null, PDFDestination::STRING_RETURN);
        return $file;
    }

    public static function store(Invoice $invoice)
    {
        self::$invoice = $invoice;
        $file = self::file();
        $isExists = Storage::exists($invoice->path);
        if ($isExists) Storage::delete($invoice->path);
        Storage::put($invoice->path, $file, 'public');
    }
}
