<?php

namespace App\Http\Controllers\Helpers;

use App\Utils\Utils;
use App\Notification;
use App\Events\NotificationCreated;

class NotificationHelper
{
    public static function create($service, $message)
    {
        $notification = [ 'service' => $service, 'message' => $message ];
        $validator = Utils::validate($notification, [
            'service' => 'required|string|max:25',
            'message' => 'required|string|max:100',
        ]);
        if ($validator->failed) return;
        $notification = Notification::create([
            'service' => $notification['service'],
            'message' => $notification['message']
        ]);
        broadcast(new NotificationCreated($notification));
    }

    public static function fuzzy($keyword)
    {
        $search = '%'.$keyword.'%';
        $notifications = Notification::where('service', 'like', $search)
            ->orWhere('message', 'like', $search)
            ->orWhere('created_at', 'like', $search)
            ->paginate(20)
            ->appends(['keyword' => $keyword]);
        return $notifications;
    }
}
