<?php

namespace App\Http\Controllers\Helpers;

use App\Invoice;
use PayPal\Api\Payer;
use PayPal\Api\Amount;
use PayPal\Api\Payment;
use PayPal\Rest\ApiContext;
use PayPal\Api\Transaction;
use PayPal\Api\RedirectUrls;
use PayPal\Api\PaymentExecution;
use PayPal\Auth\OAuthTokenCredential;

class PaypalHelper
{
    public static function paypal()
    {
        $clientId = config('payment.paypal.client_id');
        $secret = config('payment.paypal.secret');
        $credential = new OAuthTokenCredential($clientId, $secret);
        $context = new ApiContext($credential);
        $context->setConfig(['mode' => 'live']);
        return $context;
    }

    public static function payments () {
        $params = ['count' => 1, 'start_index' => 0];
        $payments = Payment::all($params, self::paypal());
        return $payments;
    }

    public static function payment(Invoice $invoice)
    {
        $payer = new Payer();
        $amount = new Amount();
        $transaction = new Transaction();
        $redirectUrls = new RedirectUrls();
        $payment = new Payment();
        $payer->setPaymentMethod('paypal');
        $amount->setTotal($invoice->milestone->budget)
            ->setCurrency('USD');
        $transaction->setAmount($amount);
        $redirectUrl = $invoice->pay;
        $redirectUrls->setReturnUrl($redirectUrl)
            ->setCancelUrl($redirectUrl);
        $payment->setIntent('sale')
            ->setPayer($payer)
            ->setTransactions([$transaction])
            ->setRedirectUrls($redirectUrls);
        return $payment;
    }

    public static function execute($paymentId, $payerId)
    {
        $context = self::paypal();
        $payment = Payment::get($paymentId, $context);
        $execution = new PaymentExecution();
        $execution->setPayerId($payerId);
        $payment->execute($execution, $context);
    }
}
