<?php

namespace App\Http\Controllers;

use App\Invoice;
use App\Milestone;
use App\Utils\Utils;
use Illuminate\Http\Request;
use App\Mail\InvoiceCreated;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Helpers\InvoiceStorageHelper as InvoiceStorage;

class InvoiceController extends Controller
{
    public function create(Request $request)
    {
        $validator = Utils::validate($request->all(), [
            'milestone_id' => 'required|exists:milestones,id|unique:invoices,milestone_id'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $milestone = Milestone::findOrFail($request->milestone_id);
        $invoiceStorage = new InvoiceStorage;
        $invoice = $milestone->invoice()->create([
            'uuid' => $invoiceStorage->id,
            'status' => 'created',
            'path' => $invoiceStorage->path(),
            'url' => $invoiceStorage->url(),
            'pay' => $invoiceStorage->pay()
        ]);
        InvoiceStorage::store($invoice);
        $invoice->milestone = $invoice->milestone;
        return Utils::response($invoice, 201);
    }

    public function paginate()
    {
        $invoices = Invoice::with('milestone.project')
            ->orderBy('id', 'desc')
            ->paginate(20);
        return Utils::response($invoices);
    }

    public function trash()
    {
        $invoices = Invoice::onlyTrashed()
            ->with('milestone.project')
            ->orderBy('deleted_at', 'desc')
            ->paginate(20);
        return Utils::response($invoices);
    }

    public function collect($uuid)
    {
        $invoice = Invoice::with('milestone.project')
            ->where('uuid', $uuid)
            ->firstOrFail();
        return Utils::response($invoice);
    }

    public function delete(Invoice $invoice)
    {
        $invoice->delete();
    }

    public function restore($id)
    {
        $invoice = Invoice::onlyTrashed()->findOrFail($id);
        $invoice->restore();
    }

    public function send(Invoice $invoice)
    {
        Mail::to($invoice->milestone->project->client)
            ->send(new InvoiceCreated($invoice));
        $isCreated = $invoice->status === 'created';
        if ($isCreated) {
            $invoice->update([ 'status' => 'sent' ]);
        }
        return Utils::response($invoice);
    }
}
