<?php

namespace App\Http\Controllers;

use App\Project;
use App\Milestone;
use App\Utils\Utils;
use Illuminate\Http\Request;

class MilestoneController extends Controller
{
    public function create(Request $request)
    {
        $validator = Utils::validate($request->all(), [
            'project_id' => 'required|integer|exists:projects,id',
            'title' => 'required|string|max:50',
            'description' => 'required|string|max:150',
            'budget' => 'required|numeric',
            'start_at' => 'required|date',
            'finish_at' => 'required|date|after:start_at'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $project = Project::findOrFail($request->project_id);
        $milestone = $project->milestones()->create([
            'title' => $request->title,
            'description' => $request->description,
            'budget' => $request->budget,
            'start_at' => $request->start_at,
            'finish_at' => $request->finish_at
        ]);
        $milestone->project = $milestone->project;
        return Utils::response($milestone, 201);
    }

    public function paginate()
    {
        $milestones = Milestone::with('project')
            ->orderBy('id', 'desc')
            ->paginate(20);
        return Utils::response($milestones);
    }

    public function trash()
    {
        $milestones = Milestone::onlyTrashed()
            ->with('project')
            ->orderBy('deleted_at', 'desc')
            ->paginate(20);
        return Utils::response($milestones);
    }

    public function read(Milestone $milestone)
    {
        $milestone->project = $milestone->project;
        return Utils::response($milestone);
    }

    public function edit(Request $request, Milestone $milestone)
    {
        $validator = Utils::validate($request->all(), [
            'title' => 'required|string|max:50',
            'description' => 'required|string|max:150',
            'budget' => 'required|numeric',
            'start_at' => 'required|date',
            'finish_at' => 'required|date|after:start_at'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $milestone->update([
            'title' => $request->title,
            'description' => $request->description,
            'budget' => $request->budget,
            'start_at' => $request->start_at,
            'finish_at' => $request->finish_at
        ]);
        $milestone->project = $milestone->project;
        return Utils::response($milestone);
    }

    public function delete(Milestone $milestone)
    {
        $milestone->delete();
    }

    public function restore($id)
    {
        $milestone = Milestone::onlyTrashed()->findOrFail($id);
        $milestone->restore();
    }
}
