<?php

namespace App\Http\Controllers;

use App\Note;
use App\Utils\Utils;
use Illuminate\Http\Request;

class NoteController extends Controller
{
    public function create(Request $request)
    {
        $validator = Utils::validate($request->all(), [
            'title' => 'required|string|max:50',
            'content' => 'required|string|max:5000'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $note = Note::create([
            'title' => $request->title,
            'content' => $request->content
        ]);
        return Utils::response($note, 201);
    }

    public function paginate()
    {
        $notes = Note::orderBy('id', 'desc')->paginate(20);
        return Utils::response($notes);
    }

    public function trash()
    {
        $notes = Note::onlyTrashed()
            ->orderBy('deleted_at', 'desc')
            ->paginate(20);
        return Utils::response($notes);
    }

    public function read(Note $note)
    {
        return Utils::response($note);
    }

    public function edit(Request $request, Note $note)
    {
        $validator = Utils::validate($request->all(), [
            'title' => 'required|string|max:50',
            'content' => 'required|string|max:5000'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $note->update([
            'title' => $request->title,
            'content' => $request->content
        ]);
        return Utils::response($note);
    }

    public function delete(Note $note)
    {
        $note->delete();
    }

    public function restore($id)
    {
        $note = Note::onlyTrashed()->findOrFail($id);
        $note->restore();
    }
}
