<?php

namespace App\Http\Controllers;

use App\Invoice;
use App\Utils\Utils;
use Illuminate\Http\Request;
use PayPal\Exception\PayPalConnectionException;
use App\Http\Controllers\Helpers\PaypalHelper as Paypal;

class PaypalController extends Controller
{
    public function create(Request $request)
    {
        $validator = Utils::validate($request->all(), [
            'uuid' => 'required|uuid|exists:invoices,uuid',
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $invoice = Invoice::where('uuid', $request->uuid)
            ->where('status', '!=', 'paid')
            ->firstOrFail();
        try {
            $payment = Paypal::payment($invoice);
            $payment->create(Paypal::paypal());
            return Utils::response($payment->getApprovalLink());
        } catch (PayPalConnectionException $e) {
            $message = $e->getMessage();
            return Utils::error($message, 500);
        }
    }
}
