<?php

namespace App\Http\Controllers;

use App\Reminder;
use App\Utils\Utils;
use Illuminate\Http\Request;

class ReminderController extends Controller
{
    public function create(Request $request)
    {
        $validator = Utils::validate($request->all(), [
            'title' => 'required|string|max:50',
            'message' => 'required|string|max:150',
            'scheduled_at' => 'required|date|after:now'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $reminder = Reminder::create([
            'title' => $request->title,
            'message' => $request->message,
            'scheduled_at' => $request->scheduled_at
        ]);
        return Utils::response($reminder, 201);
    }

    public function paginate()
    {
        $reminders = Reminder::where('is_completed', false)
            ->orderBy('id', 'desc')
            ->paginate(20);
        return Utils::response($reminders);
    }

    public function scheduled()
    {
        $reminders = Reminder::where('is_completed', false)
            ->where('scheduled_at', '<=', Utils::now())
            ->orderBy('scheduled_at', 'desc')
            ->get();
        return Utils::response($reminders);
    }

    public function completed()
    {
        $reminders = Reminder::where('is_completed', true)
            ->orderBy('completed_at', 'desc')
            ->paginate(20);
        return Utils::response($reminders);
    }

    public function trash()
    {
        $reminders = Reminder::onlyTrashed()
            ->orderBy('deleted_at', 'desc')
            ->paginate(20);
        return Utils::response($reminders);
    }

    public function read(Reminder $reminder)
    {
        return Utils::response($reminder);
    }

    public function edit(Request $request, Reminder $reminder)
    {
        $validator = Utils::validate($request->all(), [
            'title' => 'required|string|max:50',
            'message' => 'required|string|max:150',
            'scheduled_at' => 'required|date|after:now'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $reminder->update([
            'title' => $request->title,
            'message' => $request->message,
            'scheduled_at' => $request->scheduled_at
        ]);
        return Utils::response($reminder);
    }

    public function complete(Reminder $reminder)
    {
        $reminder->update([
            'is_completed' => true,
            'completed_at' => Utils::now()
        ]);
    }

    public function rollback(Reminder $reminder)
    {
        $reminder->update([
            'is_completed' => false,
            'completed_at' => null
        ]);
    }

    public function delete(Reminder $reminder)
    {
        $reminder->delete();
    }

    public function restore($id)
    {
        $reminder = Reminder::onlyTrashed()->findOrFail($id);
        $reminder->restore();
    }
}
