<?php

namespace App\Http\Controllers;

use App\Note;
use App\Todo;
use App\File;
use App\Client;
use App\Project;
use App\Payment;
use App\Invoice;
use App\Reminder;
use App\Milestone;
use App\Utils\Utils;
use Illuminate\Support\Carbon;

class ReportController extends Controller
{
    public function counts()
    {
        function payment () {
            $total = Payment::with('invoice.milestone')
                ->where('status', 'success')
                ->get()
                ->sum('invoice.milestone.budget');
            $total = round($total, 2);
            return $total;
        }
        $counts = [
            'todo' => Todo::where('is_done', 0)->count(),
            'reminder' => Reminder::where('is_completed', 0)->count(),
            'note' => Note::count(),
            'client' => Client::count(),
            'project' => Project::count(),
            'milestone' => Milestone::count(),
            'file' => File::count(),
            'invoice' => Invoice::count(),
            'payment' => payment()
        ];
        return Utils::response($counts);
    }

    public function chart()
    {
        $start = Carbon::now()->subDays(7);
        $now = Carbon::now();
        $query = [$start, $now];
        $chart = [
            'client' => Client::whereBetween('created_at', $query)->get(),
            'project' => Project::whereBetween('created_at', $query)->get(),
            'milestone' => Milestone::whereBetween('created_at', $query)->get(),
            'invoice' => Invoice::whereBetween('created_at', $query)->get(),
            'payment' => Payment::whereBetween('created_at', $query)->get()
        ];
        return Utils::response($chart);
    }
}
