<?php

namespace App\Http\Controllers;

use App\Todo;
use App\Utils\Utils;
use Illuminate\Http\Request;

class TodoController extends Controller
{
    public function create(Request $request)
    {
        $validator = Utils::validate($request->all(), [
            'task' => 'required|string|max:125'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $position = Todo::count();
        $todo = Todo::create([
            'task' => $request->task,
            'position' => $position,
        ]);
        return Utils::response($todo, 201);
    }

    public function list()
    {
        $todos = Todo::where('is_done', false)
            ->orderBy('position', 'desc')
            ->get();
        return Utils::response($todos);
    }

    public function completed()
    {
        $todos = Todo::where('is_done', true)
            ->orderBy('position', 'desc')
            ->get();
        return Utils::response($todos);
    }

    public function trash()
    {
        $todos = Todo::onlyTrashed()
            ->orderBy('deleted_at', 'desc')
            ->paginate(20);
        return Utils::response($todos);
    }

    public function reorder(Request $request)
    {
        $validator = Utils::validate($request->all(), [
            'order' => 'required|array',
            'order.*' => 'integer'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $todos = Todo::all();
        $order = array_reverse($request->order);
        foreach ($order as $position => $id) {
            $todo = $todos->find($id);
            if (!$todo) continue;
            $todo->update([ 'position' => $position ]);
        }
    }

    public function complete(Todo $todo)
    {
        $todo->update([
            'is_done' => true,
            'done_at' => Utils::now()
        ]);
    }

    public function rollback(Todo $todo)
    {
        $todo->update([
            'is_done' => false,
            'done_at' => null
        ]);
    }

    public function delete(Todo $todo)
    {
        $todo->delete();
    }

    public function restore($id)
    {
        $todo = Todo::onlyTrashed()->findOrFail($id);
        $todo->restore();
    }
}
