<?php

namespace App\Http\Controllers;

use App\User;
use App\Utils\Utils;
use Illuminate\Http\Request;
use App\Http\Controllers\Helpers\AvatarStorageHelper as AvatarStorage;

class UserController extends Controller
{
    public function create(Request $request)
    {
        $validator = Utils::validate($request->all(), [
            'name' => 'required|string|max:50',
            'designation' => 'required|string|max:50',
            'email' => 'required|string|email:rfc,dns|max:50',
            'phone' => 'required|string|max:25',
            'site' => 'required|string|max:50',
            'address' => 'required|string|max:100',
            'avatar' => 'required|array',
            'avatar.uri' => 'required|string',
            'avatar.extension' => 'required|string|max:20',
            'password' => 'required|string|min:6|max:25'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $users = User::count();
        if ($users) {
            return Utils::error('Already created');
        }
        $avatar = new AvatarStorage($request->avatar);
        $password = Utils::hash($request->password);
        $user = User::create([
            'name' => $request->name,
            'designation' => $request->designation,
            'email' => $request->email,
            'phone' => $request->phone,
            'site' => $request->site,
            'address' => $request->address,
            'avatar' => $avatar->url(),
            'password' => $password,
            'api_token' => Utils::uuid()
        ]);
        return Utils::response($user, 201);
    }

    public function read()
    {
        $user = User::firstOrFail();
        return Utils::response($user);
    }

    public function edit(Request $request)
    {
        $validator = Utils::validate($request->all(), [
            'name' => 'required|string|max:50',
            'designation' => 'required|string|max:50',
            'phone' => 'required|string|max:25',
            'site' => 'required|string|max:50',
            'address' => 'required|string|max:100',
            'avatar' => 'nullable|array',
            'avatar.uri' => 'required_with:avatar|string',
            'avatar.extension' => 'required_with:avatar|string|max:20'
        ]);
        if ($validator->failed) {
            return $validator->error();
        }
        $user = User::firstOrFail();
        $user->update([
            'name' => $request->name,
            'designation' => $request->designation,
            'phone' => $request->phone,
            'site' => $request->site,
            'address' => $request->address
        ]);
        if ($request->avatar) {
            $avatar = new AvatarStorage($request->avatar);
            $user->update([ 'avatar' => $avatar->url() ]);
        }
        return Utils::response($user);
    }
}
