<?php

namespace App\Http\Middleware;

use Closure;
use App\Http\Controllers\Helpers\ActivityHelper as Activity;
use Illuminate\Support\Arr;

class ActivityLogger
{
    private $response;
    private $request;

    private function registry()
    {
        return [
            'ServerController@verify' => [
                'service' => 'server',
                'log' => 'Verified'
            ],
            'LicenseController@verify' => [
                'service' => 'license',
                'log' => 'Verified'
            ],
            'UserController@create' => [
                'service' => 'user',
                'log' => 'Created'
            ],
            'AuthController@login' => [
                'service' => 'auth',
                'log' => 'Logged in'
            ],
            'AuthController@logout' => [
                'service' => 'auth',
                'log' => 'Logged out'
            ],
            'TodoController@create' => [
                'service' => 'todo',
                'log' => 'Created {task}'
            ],
            'TodoController@reorder' => [
                'service' => 'todo',
                'log' => 'Reordered {task}'
            ],
            'TodoController@edit' => [
                'service' => 'todo',
                'log' => 'Edited {task}'
            ],
            'TodoController@complete' => [
                'service' => 'todo',
                'log' => 'Completed {task}'
            ],
            'TodoController@rollback' => [
                'service' => 'todo',
                'log' => 'Rollbacked {task}'
            ],
            'TodoController@delete' => [
                'service' => 'todo',
                'log' => 'Deleted {task}'
            ],
            'TodoController@restore' => [
                'service' => 'todo',
                'log' => 'Restored {task}'
            ],
            'ReminderController@create' => [
                'service' => 'reminder',
                'log' => 'Created {title}'
            ],
            'ReminderController@edit' => [
                'service' => 'reminder',
                'log' => 'Edited {title}'
            ],
            'ReminderController@complete' => [
                'service' => 'reminder',
                'log' => 'Completed {title}'
            ],
            'ReminderController@rollback' => [
                'service' => 'reminder',
                'log' => 'Rollbacked {title}'
            ],
            'ReminderController@delete' => [
                'service' => 'reminder',
                'log' => 'Deleted {title}'
            ],
            'ReminderController@restore' => [
                'service' => 'reminder',
                'log' => 'Restored {title}'
            ],
            'NoteController@create' => [
                'service' => 'note',
                'log' => 'Created {title}'
            ],
            'NoteController@edit' => [
                'service' => 'note',
                'log' => 'Edited {title}'
            ],
            'NoteController@delete' => [
                'service' => 'note',
                'log' => 'Deleted {title}'
            ],
            'NoteController@restore' => [
                'service' => 'note',
                'log' => 'Restored {title}'
            ],
            'ClientController@create' => [
                'service' => 'client',
                'log' => 'Created {name}'
            ],
            'ClientController@edit' => [
                'service' => 'client',
                'log' => 'Edited {name}'
            ],
            'ClientController@delete' => [
                'service' => 'client',
                'log' => 'Deleted {name}'
            ],
            'ClientController@restore' => [
                'service' => 'client',
                'log' => 'Restored {name}'
            ],
            'ProjectController@create' => [
                'service' => 'project',
                'log' => 'Created {title}'
            ],
            'ProjectController@edit' => [
                'service' => 'project',
                'log' => 'Edited {title}'
            ],
            'ProjectController@delete' => [
                'service' => 'project',
                'log' => 'Deleted {title}'
            ],
            'ProjectController@restore' => [
                'service' => 'project',
                'log' => 'Restored {title}'
            ],
            'MilestoneController@create' => [
                'service' => 'milestone',
                'log' => 'Created {title}'
            ],
            'MilestoneController@edit' => [
                'service' => 'milestone',
                'log' => 'Edited {title}'
            ],
            'MilestoneController@delete' => [
                'service' => 'milestone',
                'log' => 'Deleted {title}'
            ],
            'MilestoneController@restore' => [
                'service' => 'milestone',
                'log' => 'Restored {title}'
            ],
            'FileController@create' => [
                'service' => 'file',
                'log' => 'Created {title}'
            ],
            'FileController@share' => [
                'service' => 'file',
                'log' => 'Shared {title}'
            ],
            'FileController@edit' => [
                'service' => 'file',
                'log' => 'Edited {title}'
            ],
            'FileController@delete' => [
                'service' => 'file',
                'log' => 'Deleted {title}'
            ],
            'FileController@restore' => [
                'service' => 'file',
                'log' => 'Restored {title}'
            ],
            'InvoiceController@create' => [
                'service' => 'invoice',
                'log' => 'Created #{id}'
            ],
            'InvoiceController@send' => [
                'service' => 'invoice',
                'log' => 'Sent #{id}'
            ],
            'InvoiceController@delete' => [
                'service' => 'invoice',
                'log' => 'Deleted #{id}'
            ],
            'InvoiceController@restore' => [
                'service' => 'invoice',
                'log' => 'Restored #{id}'
            ],
            'UserController@edit' => [
                'service' => 'settings',
                'log' => 'Edited'
            ],
            'AccountController@edit' => [
                'service' => 'account',
                'log' => 'Edited'
            ],
            'PaymentController@create' => [
                'service' => 'payment',
                'log' => 'Created {pay_id}'
            ],
            'PaymentController@cancel' => [
                'service' => 'payment',
                'log' => 'Canceled {token}'
            ],
        ];
    }

    private function topic($key)
    {
        $response = $this->response;
        $isApplicable = Arr::has($response, $key);
        if ($isApplicable) return $response[$key];
    }

    private function controller()
    {
        $route = $this->request->route();
        $controllerPath = $route->action['controller'];
        $controller = str_replace('App\\Http\\Controllers\\', '', $controllerPath);
        return $controller;
    }

    private function replacer($activity)
    {
        $regex = '/{(.*?)}/';
        $log = $activity['log'];
        preg_match_all($regex, $log, $matches);
        $variable = $matches[0] ? $matches[0][0] : null;
        $key = $matches[1] ? $matches[1][0] : null;
        $topic = $this->topic($key);
        if ($key && !$topic) return;
        $log = str_replace($variable, $topic, $log);
        $activity['log'] = $log;
        return $activity;
    }

    private function activity()
    {
        $controller = $this->controller();
        $activities = $this->registry();
        $isApplicable = Arr::has($activities, $controller);
        if (!$isApplicable) return;
        $activity = Arr::get($activities, $controller);
        $activity = $this->replacer($activity);
        return $activity;
    }

    public function canLog($response, $activity)
    {
        $isSuccess = in_array($response->status(), [200, 201]);
        $canLog = $isSuccess && $activity;
        return $canLog;
    }

    public function handle($request, Closure $next)
    {
        $response = $next($request);
        $this->response = $response->original;
        if ($response->exception) return $response;
        $this->request = $request;
        $activity = $this->activity();
        if ($this->canLog($response, $activity)) {
            Activity::create($activity);
        }
        return $response;
    }
}
