import Chart from 'chart.js'
import $ from 'jquery'
import _ from 'lodash'
import moment from 'moment'
import ReportStore from './StoreServices/ReportStoreService'

export default class ChartService {
  constructor (selector) {
    this.selector = selector
  }

  counts (weekly) {
    const format = _.map(weekly, item => moment(item.created_at).format('ddd'))
    weekly = _.countBy(format, day => day)
    const counts = _.map(_.rangeRight(7), data => {
      const day = moment().subtract(data, 'days').format('ddd')
      var visits = weekly[day]
      if (!visits) visits = 0
      return visits
    })
    return counts
  }

  days () {
    const days = _.map(_.rangeRight(7), data => {
      const day = moment().subtract(data, 'days').format('ddd')
      return day
    })
    return days
  }

  data () {
    const chart = ReportStore.collectChart()
    const data = {
      client: this.counts(chart.client),
      project: this.counts(chart.project),
      milestone: this.counts(chart.milestone),
      invoice: this.counts(chart.invoice),
      payment: this.counts(chart.payment)
    }
    return data
  }

  init () {
    const dom = $(this.selector)
    new Chart(dom, {
      type: 'line',
      data: {
        labels: this.days(),
        datasets: [
          {
            label: 'Client',
            data: this.data().client,
            fill: false,
            borderColor: '#a4036f',
            backgroundColor: '#a4036f',
            borderWidth: 1
          },
          {
            label: 'Project',
            data: this.data().project,
            fill: false,
            borderColor: '#2c908f',
            backgroundColor: '#2c908f',
            borderWidth: 1
          },
          {
            label: 'Milestone',
            data: this.data().milestone,
            fill: false,
            borderColor: '#42db93',
            backgroundColor: '#42db93',
            borderWidth: 1
          },
          {
            label: 'Invoice',
            data: this.data().invoice,
            fill: false,
            borderColor: '#efea5a',
            backgroundColor: '#efea5a',
            borderWidth: 1
          },
          {
            label: 'Payment',
            data: this.data().payment,
            fill: false,
            borderColor: '#f29e4c',
            backgroundColor: '#f29e4c',
            borderWidth: 1
          }
        ]
      },
      options: {
        responsive: true,
        maintainAspectRatio: false,
        scales: {
          xAxes: [{ gridLines: { display: false }, ticks: { fontSize: 8.5 } }],
          yAxes: [{ gridLines: { display: false }, ticks: { fontSize: 8.5 } }]
        },
        legend: { labels: { fontSize: 9 } }
      }
    })
  }
}
