import axios from 'axios'
import Alert from '../AlertService'
import AuthToken from '../AuthTokenService'
import Progress from '../ProgressService'

export default class HttpService {
  static instance () {
    const instance = axios.create({
      baseURL: `${process.env.MIX_APP_URL}/api`,
      headers: {
        Authorization: `Bearer ${AuthToken.collect()}`
      }
    })
    return instance
  }

  static async get (url, params) {
    try {
      Progress.start()
      const config = { params }
      const response = await this.instance().get(url, config)
      Progress.end()
      return response.data
    } catch (error) {
      Progress.end()
      const data = error.response.data
      if (data) Alert.error(data.message)
    }
  }

  static async post (url, data) {
    try {
      Progress.start()
      const response = await this.instance().post(url, data)
      Progress.end()
      return response.data
    } catch (error) {
      Progress.end()
      const data = error.response.data
      if (data) Alert.error(data.message)
    }
  }

  static async put (url, data) {
    try {
      Progress.start()
      const response = await this.instance().put(url, data)
      Progress.end()
      return response.data
    } catch (error) {
      Progress.end()
      const data = error.response.data
      if (data) Alert.error(data.message)
    }
  }

  static async delete (url) {
    try {
      Progress.start()
      const response = await this.instance().delete(url)
      Progress.end()
      return response.data
    } catch (error) {
      Progress.end()
      const data = error.response.data
      if (data) Alert.error(data.message)
    }
  }
}
