import _ from 'lodash'
import Masonry from 'masonry-layout'
import $ from 'jquery'

export default class MasonryService {
  constructor (component) {
    this.component = component
    this.init()
  }

  isSmallScreen () {
    const deviceWidth = document.documentElement.clientWidth
    const isSmallScreen = deviceWidth <= 425
    return isSmallScreen
  }

  columnWidth (grid) {
    const domWidth = grid.outerWidth()
    const columnWidth = this.isSmallScreen() ? domWidth : ((domWidth - 40) / 3)
    return columnWidth
  }

  options (grid) {
    const options = {
      itemSelector: '.grid-item',
      columnWidth: this.columnWidth(grid),
      gutter: 20,
      percentPosition: 'true',
      fitWidth: true
    }
    return options
  }

  nextTick () {
    return Promise.resolve()
  }

  async init () {
    await this.nextTick()
    const grid = $(`#${this.component}-grid`)
    const hasGrid = !_.isEmpty(grid)
    if (!hasGrid) return
    new Masonry(grid[0], this.options(grid))
  }
}
