import ClientHttp from '../HttpServices/ClientHttpService'
import ClientStore from '../StoreServices/ClientStoreService'
import ClientModal from '../ModalServices/ClientModalService'
import ClientMasonry from '../MasonryServices/ClientMasonryService'

export default class ClientRequestService {
  static async create (client) {
    client = await ClientHttp.create(client)
    if (!client) return
    ClientStore.addPaginate(client)
    ClientModal.create().close()
    ClientMasonry.paginate()
  }

  static async paginate () {
    const paginate = await ClientHttp.paginate()
    if (!paginate) return
    ClientStore.replacePaginate(paginate)
    ClientMasonry.paginate()
  }

  static async shortlist () {
    const shortlist = await ClientHttp.shortlist()
    if (!shortlist) return
    ClientStore.replaceShortlist(shortlist)
  }

  static async trash () {
    const trash = await ClientHttp.trash()
    if (!trash) return
    ClientStore.replaceTrash(trash)
    ClientMasonry.trash()
  }

  static async read (id) {
    const client = await ClientHttp.read(id)
    return client
  }

  static async edit (client) {
    client = await ClientHttp.edit(client)
    if (!client) return
    ClientStore.amendPaginate(client)
    ClientModal.edit().close()
  }

  static async delete (id) {
    await ClientHttp.delete(id)
    ClientStore.pullPaginate(id)
    ClientModal.delete().close()
    ClientMasonry.paginate()
  }

  static async restore (id) {
    await ClientHttp.restore(id)
    ClientStore.pullTrash(id)
    ClientModal.restore().close()
    ClientMasonry.trash()
  }

  static async nextPaginate () {
    const current = ClientStore.collectPaginate()
    if (!current.next_page_url) return
    const paginate = await ClientHttp.next(current.next_page_url)
    if (!paginate) return
    ClientStore.pushPaginate(paginate)
  }

  static async nextTrash () {
    const current = ClientStore.collectTrash()
    if (!current.next_page_url) return
    const trash = await ClientHttp.next(current.next_page_url)
    if (!trash) return
    ClientStore.pushTrash(trash)
  }
}
