import FileHttp from '../HttpServices/FileHttpService'
import FileStore from '../StoreServices/FileStoreService'
import FileMasonry from '../MasonryServices/FileMasonryService'
import FileModal from '../ModalServices/FileModalService'
import Alert from '../AlertService'

export default class FileRequestService {
  static async create (file) {
    file = await FileHttp.create(file)
    if (!file) return
    FileStore.addPaginate(file)
    FileModal.create().close()
    FileMasonry.paginate()
  }

  static async paginate () {
    const paginate = await FileHttp.paginate()
    if (!paginate) return
    FileStore.replacePaginate(paginate)
    FileMasonry.paginate()
  }

  static async trash () {
    const trash = await FileHttp.trash()
    if (!trash) return
    FileStore.replaceTrash(trash)
    FileMasonry.trash()
  }

  static async read (id) {
    const file = await FileHttp.read(id)
    return file
  }

  static async edit (file) {
    file = await FileHttp.edit(file)
    if (!file) return
    FileStore.amendPaginate(file)
    FileModal.edit().close()
  }

  static async delete (id) {
    await FileHttp.delete(id)
    FileStore.pullPaginate(id)
    FileModal.delete().close()
    FileMasonry.paginate()
  }

  static async restore (id) {
    await FileHttp.restore(id)
    FileStore.pullTrash(id)
    FileModal.restore().close()
    FileMasonry.trash()
  }

  static async share (share) {
    share = await FileHttp.share(share)
    if (!share) return
    Alert.success('Successfully shared file')
    FileModal.share().close()
  }

  static async collect (uuid) {
    const file = await FileHttp.collect(uuid)
    return file
  }

  static async nextPaginate () {
    const current = FileStore.collectPaginate()
    if (!current.next_page_url) return
    const paginate = await FileHttp.next(current.next_page_url)
    if (!paginate) return
    FileStore.pushPaginate(paginate)
  }

  static async nextTrash () {
    const current = FileStore.collectTrash()
    if (!current.next_page_url) return
    const trash = await FileHttp.next(current.next_page_url)
    if (!trash) return
    FileStore.pushTrash(trash)
  }
}
