import InvoiceHttp from '../HttpServices/InvoiceHttpService'
import InvoiceStore from '../StoreServices/InvoiceStoreService'
import InvoiceMasonry from '../MasonryServices/InvoiceMasonryService'
import InvoiceModal from '../ModalServices/InvoiceModalService'
import Alert from '../AlertService'

export default class InvoiceRequestService {
  static async create (invoice) {
    invoice = await InvoiceHttp.create(invoice)
    if (!invoice) return
    InvoiceStore.addPaginate(invoice)
    InvoiceModal.create().close()
    InvoiceMasonry.paginate()
    return invoice
  }

  static async paginate () {
    const paginate = await InvoiceHttp.paginate()
    if (!paginate) return
    InvoiceStore.replacePaginate(paginate)
    InvoiceMasonry.paginate()
  }

  static async trash () {
    const trash = await InvoiceHttp.trash()
    if (!trash) return
    InvoiceStore.replaceTrash(trash)
    InvoiceMasonry.trash()
  }

  static async read (uuid) {
    const invoice = await InvoiceHttp.read(uuid)
    return invoice
  }

  static async delete (id) {
    await InvoiceHttp.delete(id)
    InvoiceStore.pullPaginate(id)
    InvoiceModal.delete().close()
    InvoiceMasonry.paginate()
  }

  static async restore (id) {
    await InvoiceHttp.restore(id)
    InvoiceStore.pullTrash(id)
    InvoiceModal.restore().close()
    InvoiceMasonry.trash()
  }

  static async send (id) {
    const invoice = await InvoiceHttp.send(id)
    if (!invoice) return
    InvoiceStore.amendPaginate(invoice)
    Alert.success('Invoice sent successfully')
  }

  static async nextPaginate () {
    const current = InvoiceStore.collectPaginate()
    if (!current.next_page_url) return
    const paginate = await InvoiceHttp.next(current.next_page_url)
    if (!paginate) return
    InvoiceStore.pushPaginate(paginate)
  }

  static async nextTrash () {
    const current = InvoiceStore.collectTrash()
    if (!current.next_page_url) return
    const trash = await InvoiceHttp.next(current.next_page_url)
    if (!trash) return
    InvoiceStore.pushTrash(trash)
  }
}
