import MilestoneHttp from '../HttpServices/MilestoneHttpService'
import MilestoneStore from '../StoreServices/MilestoneStoreService'
import MilestoneModal from '../ModalServices/MilestoneModalService'
import MilestoneMasonry from '../MasonryServices/MilestoneMasonryService'

export default class MilestoneRequestService {
  static async create (milestone) {
    milestone = await MilestoneHttp.create(milestone)
    if (!milestone) return
    MilestoneStore.addPaginate(milestone)
    MilestoneModal.create().close()
    MilestoneMasonry.paginate()
  }

  static async paginate () {
    const paginate = await MilestoneHttp.paginate()
    if (!paginate) return
    MilestoneStore.replacePaginate(paginate)
    MilestoneMasonry.paginate()
  }

  static async trash () {
    const trash = await MilestoneHttp.trash()
    if (!trash) return
    MilestoneStore.replaceTrash(trash)
    MilestoneMasonry.trash()
  }

  static async read (id) {
    const milestone = await MilestoneHttp.read(id)
    return milestone
  }

  static async edit (milestone) {
    milestone = await MilestoneHttp.edit(milestone)
    if (!milestone) return
    MilestoneStore.amendPaginate(milestone)
    MilestoneModal.edit().close()
  }

  static async delete (id) {
    await MilestoneHttp.delete(id)
    MilestoneStore.pullPaginate(id)
    MilestoneModal.delete().close()
    MilestoneMasonry.paginate()
  }

  static async restore (id) {
    await MilestoneHttp.restore(id)
    MilestoneStore.pullTrash(id)
    MilestoneModal.restore().close()
    MilestoneMasonry.trash()
  }

  static async nextPaginate () {
    const current = await MilestoneStore.collectPaginate()
    if (!current.next_page_url) return
    const paginate = await MilestoneHttp.next(current.next_page_url)
    if (!paginate) return
    MilestoneStore.pushPaginate(paginate)
  }

  static async nextTrash () {
    const current = await MilestoneStore.collectTrash()
    if (!current.next_page_url) return
    const trash = await MilestoneHttp.next(current.next_page_url)
    if (!trash) return
    MilestoneStore.pushTrash(trash)
  }
}
