import NoteHttp from '../HttpServices/NoteHttpService'
import NoteStore from '../StoreServices/NoteStoreService'
import NoteMasonry from '../MasonryServices/NoteMasonryService'
import NoteModal from '../ModalServices/NoteModalService'

export default class NoteRequestService {
  static async create (note) {
    note = await NoteHttp.create(note)
    if (!note) return
    NoteStore.addPaginate(note)
    NoteModal.create().close()
    NoteMasonry.paginate()
  }

  static async paginate () {
    const paginate = await NoteHttp.paginate()
    if (!paginate) return
    NoteStore.replacePaginate(paginate)
    NoteMasonry.paginate()
  }

  static async trash () {
    const trash = await NoteHttp.trash()
    if (!trash) return
    NoteStore.replaceTrash(trash)
    NoteMasonry.trash()
  }

  static async read (id) {
    const note = await NoteHttp.read(id)
    return note
  }

  static async edit (note) {
    note = await NoteHttp.edit(note)
    if (!note) return
    NoteStore.amendPaginate(note)
    NoteModal.edit().close()
  }

  static async delete (id) {
    await NoteHttp.delete(id)
    NoteStore.pullPaginate(id)
    NoteModal.delete().close()
    NoteMasonry.paginate()
  }

  static async restore (id) {
    await NoteHttp.restore(id)
    NoteStore.pullTrash(id)
    NoteModal.restore().close()
    NoteMasonry.trash()
  }

  static async nextPaginate () {
    const current = NoteStore.collectPaginate()
    if (!current.next_page_url) return
    const paginate = await NoteHttp.next(current.next_page_url)
    if (!paginate) return
    NoteStore.pushPaginate(paginate)
  }

  static async nextTrash () {
    const current = NoteStore.collectTrash()
    if (!current.next_page_url) return
    const paginate = await NoteHttp.next(current.next_page_url)
    if (!paginate) return
    NoteStore.pushTrash(paginate)
  }
}
