import ProjectHttp from '../HttpServices/ProjectHttpService'
import ProjectStore from '../StoreServices/ProjectStoreService'
import ProjectModal from '../ModalServices/ProjectModalService'
import ProjectMasonry from '../MasonryServices/ProjectMasonryService'

export default class ProjectRequestService {
  static async create (project) {
    project = await ProjectHttp.create(project)
    if (!project) return
    ProjectStore.addPaginate(project)
    ProjectModal.create().close()
    ProjectMasonry.paginate()
  }

  static async paginate () {
    const paginate = await ProjectHttp.paginate()
    if (!paginate) return
    ProjectStore.replacePaginate(paginate)
    ProjectMasonry.paginate()
  }

  static async shortlist () {
    const shortlist = await ProjectHttp.shortlist()
    if (!shortlist) return
    ProjectStore.replaceShortlist(shortlist)
  }

  static async trash () {
    const trash = await ProjectHttp.trash()
    if (!trash) return
    ProjectStore.replaceTrash(trash)
    ProjectMasonry.trash()
  }

  static async read (id) {
    const project = await ProjectHttp.read(id)
    return project
  }

  static async edit (project) {
    project = await ProjectHttp.edit(project)
    if (!project) return
    ProjectStore.amendPaginate(project)
    ProjectModal.edit().close()
  }

  static async delete (id) {
    await ProjectHttp.delete(id)
    ProjectStore.pullPaginate(id)
    ProjectModal.delete().close()
    ProjectMasonry.paginate()
  }

  static async restore (id) {
    await ProjectHttp.restore(id)
    ProjectStore.pullTrash(id)
    ProjectModal.restore().close()
    ProjectMasonry.trash()
  }

  static async milestones (id) {
    const milestones = await ProjectHttp.milestones(id)
    if (!milestones) return
    ProjectStore.replaceMilestones(milestones)
  }

  static async nextPaginate () {
    const current = ProjectStore.collectPaginate()
    if (!current.next_page_url) return
    const paginate = await ProjectHttp.next(current.next_page_url)
    if (!paginate) return
    ProjectStore.pushPaginate(paginate)
  }

  static async nextTrash () {
    const current = ProjectStore.collectTrash()
    if (!current.next_page_url) return
    const trash = await ProjectHttp.next(current.next_page_url)
    if (!trash) return
    ProjectStore.pushTrash(trash)
  }
}
