import ReminderHttp from '../HttpServices/ReminderHttpService'
import ReminderStore from '../StoreServices/ReminderStoreService'
import ReminderMasonry from '../MasonryServices/ReminderMasonryService'
import ReminderModal from '../ModalServices/ReminderModalService'

export default class ReminderRequestService {
  static async create (reminder) {
    reminder = await ReminderHttp.create(reminder)
    if (!reminder) return
    ReminderStore.addPaginate(reminder)
    ReminderModal.create().close()
    ReminderMasonry.paginate()
  }

  static async paginate () {
    const paginate = await ReminderHttp.paginate()
    if (!paginate) return
    ReminderStore.replacePaginate(paginate)
    ReminderMasonry.paginate()
  }

  static async scheduled () {
    const scheduled = await ReminderHttp.scheduled()
    if (!scheduled) return
    ReminderStore.replaceScheduled(scheduled)
  }

  static async completed () {
    const completed = await ReminderHttp.completed()
    if (!completed) return
    ReminderStore.replaceCompleted(completed)
    ReminderMasonry.completed()
  }

  static async trash () {
    const trash = await ReminderHttp.trash()
    if (!trash) return
    ReminderStore.replaceTrash(trash)
    ReminderMasonry.trash()
  }

  static async read (id) {
    const reminder = await ReminderHttp.read(id)
    return reminder
  }

  static async edit (reminder) {
    reminder = await ReminderHttp.edit(reminder)
    if (!reminder) return
    ReminderStore.amendPaginate(reminder)
    ReminderModal.edit().close()
  }

  static async complete (id) {
    await ReminderHttp.complete(id)
    ReminderStore.pullPaginate(id)
    ReminderModal.complete().close()
    ReminderMasonry.paginate()
  }

  static async rollback (id) {
    await ReminderHttp.rollback(id)
    ReminderStore.pullCompleted(id)
    ReminderModal.rollback().close()
    ReminderMasonry.completed()
  }

  static async delete (id) {
    await ReminderHttp.delete(id)
    ReminderStore.pullPaginate(id)
    ReminderStore.pullCompleted(id)
    ReminderModal.delete().close()
    ReminderMasonry.paginate()
    ReminderMasonry.completed()
  }

  static async restore (id) {
    await ReminderHttp.restore(id)
    ReminderStore.pullTrash(id)
    ReminderModal.restore().close()
    ReminderMasonry.trash()
  }

  static async nextPaginate () {
    const current = ReminderStore.collectPaginate()
    if (!current.next_page_url) return
    const paginate = await ReminderHttp.next(current.next_page_url)
    if (!paginate) return
    ReminderStore.pushPaginate(paginate)
    ReminderMasonry.paginate()
  }

  static async nextCompleted () {
    const current = ReminderStore.collectCompleted()
    if (!current.next_page_url) return
    const completed = await ReminderHttp.next(current.next_page_url)
    if (!completed) return
    ReminderStore.pushCompleted(completed)
    ReminderMasonry.completed()
  }

  static async nextTrash () {
    const current = ReminderStore.collectTrash()
    if (!current.next_page_url) return
    const trash = await ReminderHttp.next(current.next_page_url)
    if (!trash) return
    ReminderStore.pushTrash(trash)
    ReminderMasonry.trash()
  }
}
