import TodoHttp from '../HttpServices/TodoHttpService'
import TodoStore from '../StoreServices/TodoStoreService'
import TodoModal from '../ModalServices/TodoModalService'
import TodoMasonry from '../MasonryServices/TodoMasonryService'

export default class TodoRequestService {
  static async create (task) {
    task = await TodoHttp.create(task)
    if (!task) return
    TodoStore.addList(task)
    TodoModal.create().close()
  }

  static async list () {
    const list = await TodoHttp.list()
    if (!list) return
    TodoStore.replaceList(list)
  }

  static async completed () {
    const completed = await TodoHttp.completed()
    if (!completed) return
    TodoStore.replaceCompleted(completed)
  }

  static async trash () {
    const trash = await TodoHttp.trash()
    if (!trash) return
    TodoStore.replaceTrash(trash)
    TodoMasonry.trash()
  }

  static async reorder () {
    const order = TodoStore.collectOrder()
    if (!order) return
    const reorder = { order }
    await TodoHttp.reorder(reorder)
  }

  static async complete (id) {
    await TodoHttp.complete(id)
  }

  static async rollback (id) {
    await TodoHttp.rollback(id)
  }

  static async delete (id) {
    await TodoHttp.delete(id)
    TodoStore.pullList(id)
    TodoStore.pullCompleted(id)
    TodoModal.delete().close()
  }

  static async restore (id) {
    await TodoHttp.restore(id)
    TodoStore.pullTrash(id)
    TodoModal.restore().close()
    TodoMasonry.trash()
  }

  static async nextTrash () {
    const current = TodoStore.collectTrash()
    if (!current.next_page_url) return
    const trash = await TodoHttp.next(current.next_page_url)
    if (!trash) return
    TodoStore.pushTrash(trash)
  }
}
