import _ from 'lodash'

export default {
  namespaced: true,
  state: {
    active: 'paginate',
    reminder: {},
    paginate: {
      data: [],
      next_page_url: ''
    },
    scheduled: [],
    completed: {
      data: [],
      next_page_url: ''
    },
    trash: {
      data: [],
      next_page_url: ''
    }
  },

  mutations: {
    addPaginate (state, reminder) {
      state.paginate.data.unshift(reminder)
    },
    addScheduled (state, reminder) {
      state.scheduled.push(reminder)
    },
    replaceActive (state, active) {
      state.active = active
    },
    replaceReminder (state, reminder) {
      state.reminder = reminder
    },
    replacePaginate (state, paginate) {
      state.paginate = paginate
    },
    replaceScheduled (state, scheduled) {
      state.scheduled = scheduled
    },
    replaceCompleted (state, completed) {
      state.completed = completed
    },
    replaceTrash (state, trash) {
      state.trash = trash
    },
    pushPaginate (state, paginate) {
      _.forEach(paginate.data, reminder => {
        state.paginate.data.push(reminder)
      })
      state.paginate.next_page_url = paginate.next_page_url
    },
    pushCompleted (state, completed) {
      _.forEach(completed.data, reminder => {
        state.completed.data.push(reminder)
      })
      state.completed.next_page_url = completed.next_page_url
    },
    pushTrash (state, trash) {
      _.forEach(trash.data, reminder => {
        state.trash.data.push(reminder)
      })
      state.trash.next_page_url = trash.next_page_url
    },
    amendPaginate (state, reminder) {
      const query = { id: reminder.id }
      const index = _.findIndex(state.paginate.data, query)
      state.paginate.data.splice(index, 1, reminder)
    },
    pullPaginate (state, id) {
      const reminder = _.find(state.paginate.data, { id })
      if (!reminder) return
      const index = _.findIndex(state.paginate.data, { id })
      state.paginate.data.splice(index, 1)
    },
    pullScheduled (state, id) {
      const reminder = _.find(state.scheduled, { id })
      if (!reminder) return
      const index = _.findIndex(state.scheduled, { id })
      state.scheduled.splice(index, 1)
    },
    pullCompleted (state, id) {
      const reminder = _.find(state.completed.data, { id })
      if (!reminder) return
      const index = _.findIndex(state.completed.data, { id })
      state.completed.data.splice(index, 1)
    },
    pullTrash (state, id) {
      const reminder = _.find(state.trash.data, { id })
      if (!reminder) return
      const index = _.findIndex(state.trash.data, { id })
      state.trash.data.splice(index, 1)
    }
  },

  getters: {
    active: state => state.active,
    reminder: state => state.reminder,
    paginate: state => state.paginate,
    scheduled: state => state.scheduled,
    completed: state => state.completed,
    trash: state => state.trash
  }
}
