<?php

use Illuminate\Support\Facades\Route;

Route::middleware('fresh.deployment')->group(function () {
    Route::get('setup/verify', 'SetupController@verify');
    Route::get('license/verify', 'LicenseController@verify');
    Route::post('user', 'UserController@create');
});

Route::prefix('auth')->group(function() {
    Route::post('login', 'AuthController@login');
    Route::get('logout', 'AuthController@logout');
});

Route::middleware('auth:api')->group(function () {
    Route::prefix('reports')->group(function () {
        Route::get('counts', 'ReportController@counts');
        Route::get('chart', 'ReportController@chart');
    });

    Route::prefix('todos')->group(function () {
        Route::post('', 'TodoController@create');
        Route::get('', 'TodoController@list');
        Route::get('completed', 'TodoController@completed');
        Route::get('trash', 'TodoController@trash');
        Route::put('reorder', 'TodoController@reorder');
        Route::put('{todo}/complete', 'TodoController@complete');
        Route::put('{todo}/rollback', 'TodoController@rollback');
        Route::delete('{todo}', 'TodoController@delete');
        Route::delete('trash/{todo}', 'TodoController@restore');
    });

    Route::prefix('reminders')->group(function () {
        Route::post('', 'ReminderController@create');
        Route::get('paginate', 'ReminderController@paginate');
        Route::get('scheduled', 'ReminderController@scheduled');
        Route::get('completed', 'ReminderController@completed');
        Route::get('trash', 'ReminderController@trash');
        Route::get('{reminder}', 'ReminderController@read');
        Route::put('{reminder}', 'ReminderController@edit');
        Route::put('{reminder}/complete', 'ReminderController@complete');
        Route::put('{reminder}/rollback', 'ReminderController@rollback');
        Route::delete('{reminder}', 'ReminderController@delete');
        Route::delete('trash/{reminder}', 'ReminderController@restore');
    });

    Route::prefix('notes')->group(function () {
        Route::post('', 'NoteController@create');
        Route::get('paginate', 'NoteController@paginate');
        Route::get('trash', 'NoteController@trash');
        Route::get('{note}', 'NoteController@read');
        Route::put('{note}', 'NoteController@edit');
        Route::delete('{note}', 'NoteController@delete');
        Route::delete('trash/{note}', 'NoteController@restore');
    });

    Route::prefix('clients')->group(function () {
        Route::post('', 'ClientController@create');
        Route::get('paginate', 'ClientController@paginate');
        Route::get('shortlist', 'ClientController@shortlist');
        Route::get('trash', 'ClientController@trash');
        Route::get('{client}', 'ClientController@read');
        Route::put('{client}', 'ClientController@edit');
        Route::delete('{client}', 'ClientController@delete');
        Route::delete('trash/{client}', 'ClientController@restore');
    });

    Route::prefix('projects')->group(function() {
        Route::post('', 'ProjectController@create');
        Route::get('paginate', 'ProjectController@paginate');
        Route::get('shortlist', 'ProjectController@shortlist');
        Route::get('trash', 'ProjectController@trash');
        Route::get('{project}', 'ProjectController@read');
        Route::put('{project}', 'ProjectController@edit');
        Route::delete('{project}', 'ProjectController@delete');
        Route::delete('trash/{project}', 'ProjectController@restore');
        Route::get('{project}/milestones', 'ProjectController@milestones');
    });

    Route::prefix('milestones')->group(function() {
        Route::post('', 'MilestoneController@create');
        Route::get('paginate', 'MilestoneController@paginate');
        Route::get('trash', 'MilestoneController@trash');
        Route::get('{milestone}', 'MilestoneController@read');
        Route::put('{milestone}', 'MilestoneController@edit');
        Route::delete('{milestone}', 'MilestoneController@delete');
        Route::delete('trash/{milestone}', 'MilestoneController@restore');
    });

    Route::prefix('files')->group(function () {
        Route::post('', 'FileController@create');
        Route::get('paginate', 'FileController@paginate');
        Route::get('trash', 'FileController@trash');
        Route::get('{file}', 'FileController@read');
        Route::put('{file}', 'FileController@edit');
        Route::delete('{file}', 'FileController@delete');
        Route::delete('trash/{file}', 'FileController@restore');
        Route::post('{file}/share', 'FileController@share');
    });

    Route::prefix('invoices')->group(function () {
        Route::post('', 'InvoiceController@create');
        Route::get('paginate', 'InvoiceController@paginate');
        Route::get('trash', 'InvoiceController@trash');
        Route::delete('{invoice}', 'InvoiceController@delete');
        Route::delete('trash/{invoice}', 'InvoiceController@restore');
        Route::post('{invoice}/send', 'InvoiceController@send');
    });

    Route::get('payments/paginate', 'PaymentController@paginate');

    Route::prefix('notifications')->group(function () {
        Route::get('paginate', 'NotificationController@paginate');
        Route::get('search', 'NotificationController@search');
    });

    Route::prefix('activities')->group(function () {
        Route::get('paginate', 'ActivityController@paginate');
        Route::get('search', 'ActivityController@search');
    });

    Route::prefix('user')->group(function () {
        Route::get('', 'UserController@read');
        Route::put('', 'UserController@edit');
    });

    Route::put('account', 'AccountController@edit');
});

Route::get('invoices/{uuid}', 'InvoiceController@collect');
Route::get('files/{uuid}', 'FileController@collect');
Route::post('paypal', 'PaypalController@create');

Route::prefix('payments')->group(function () {
    Route::post('', 'PaymentController@create');
    Route::post('cancel', 'PaymentController@cancel');
});

Route::get('install/verify', 'InstallController@verify');